@@ -25,6 +25,7 @@ use rustc_span::hygiene::ExpnData;
 use rustc_span::source_map::{FilePathMapping, SourceMap};
 use serde::Serialize;
 
+use crate::annotate_snippet_emitter_writer::AnnotateSnippetEmitter;
 use crate::diagnostic::IsLint;
 use crate::emitter::{
     ColorConfig, Destination, Emitter, HumanEmitter, HumanReadableErrorType, OutputTheme,
@@ -370,7 +371,6 @@ impl Diagnostic {
                 .insert(0, Diagnostic::from_sub_diagnostic(&diag.emitted_at_sub_diag(), &args, je));
         }
         let buf = BufWriter(Arc::new(Mutex::new(Vec::new())));
-        let short = je.json_rendered.short();
         let dst: Destination = AutoStream::new(
             Box::new(buf.clone()),
             match je.color_config.to_color_choice() {
@@ -378,21 +378,39 @@ impl Diagnostic {
                 choice => choice,
             },
         );
-        HumanEmitter::new(dst, je.translator.clone())
-            .short_message(short)
-            .sm(je.sm.clone())
-            .diagnostic_width(je.diagnostic_width)
-            .macro_backtrace(je.macro_backtrace)
-            .track_diagnostics(je.track_diagnostics)
-            .terminal_url(je.terminal_url)
-            .ui_testing(je.ui_testing)
-            .ignored_directories_in_source_blocks(je.ignored_directories_in_source_blocks.clone())
-            .theme(if let HumanReadableErrorType::Unicode = je.json_rendered {
-                OutputTheme::Unicode
-            } else {
-                OutputTheme::Ascii
-            })
-            .emit_diagnostic(diag, registry);
+        match je.json_rendered {
+            HumanReadableErrorType::AnnotateSnippet { short, unicode } => {
+                AnnotateSnippetEmitter::new(dst, je.translator.clone())
+                    .short_message(short)
+                    .sm(je.sm.clone())
+                    .diagnostic_width(je.diagnostic_width)
+                    .macro_backtrace(je.macro_backtrace)
+                    .track_diagnostics(je.track_diagnostics)
+                    .terminal_url(je.terminal_url)
+                    .ui_testing(je.ui_testing)
+                    .ignored_directories_in_source_blocks(
+                        je.ignored_directories_in_source_blocks.clone(),
+                    )
+                    .theme(if unicode { OutputTheme::Unicode } else { OutputTheme::Ascii })
+                    .emit_diagnostic(diag, registry)
+            }
+            HumanReadableErrorType::Default { short } => {
+                HumanEmitter::new(dst, je.translator.clone())
+                    .short_message(short)
+                    .sm(je.sm.clone())
+                    .diagnostic_width(je.diagnostic_width)
+                    .macro_backtrace(je.macro_backtrace)
+                    .track_diagnostics(je.track_diagnostics)
+                    .terminal_url(je.terminal_url)
+                    .ui_testing(je.ui_testing)
+                    .ignored_directories_in_source_blocks(
+                        je.ignored_directories_in_source_blocks.clone(),
+                    )
+                    .theme(OutputTheme::Ascii)
+                    .emit_diagnostic(diag, registry)
+            }
+        }
+
         let buf = Arc::try_unwrap(buf.0).unwrap().into_inner().unwrap();
         let buf = String::from_utf8(buf).unwrap();
 
